#!/user/bin/env python3
# -*- coding: UTF-8 -*-
# @Time : 2024/6/6 上午6:16
# @Author : 龙翔
# @File    :RedisServer.py
# @Software: PyCharm
import json
import os
import sys
import time
import uuid
import redis


class RedisQueue:
    def __init__(self, r: redis.Redis, topic: str = 'test_queue'):
        self.topic = topic
        self.r = r

    def get(self):
        # 弹出指定数量的数据
        data = self.r.rpop(self.topic)
        return data if data else None

    def put(self, value):
        if isinstance(value, list):
            self.r.lpush(self.topic, *value)
        else:
            self.r.lpush(self.topic, *[value])

    def clear(self):
        self.r.delete(self.topic)

    def size(self):
        return self.r.llen(self.topic)

    def qsize(self):
        return self.size()

    def get_mul(self, num):
        return [self.get() for _ in range(num)]

    def re_data(self):
        ch_keys = self.r.keys(f"ack_{self.topic}_*")
        for key in ch_keys:
            data = self.r.get(key)
            if data:
                q = RedisQueue(self.r, self.topic)
                t, _data = json.loads(data)
                self.r.delete(key)
                q.put(_data)
        return len(ch_keys)

    def get_all(self):
        return self.r.lrange(self.topic, 0, -1)


class RedisMQ:
    def __init__(self):
        self.switch = 1

    def start_receive(self, r, topic, callback, count=-1):
        while self.switch:
            data = r.rpop(topic)
            if data:
                ch = RedisCh(r, topic, data)
                callback(ch, data)
                if count == 1:
                    return
                continue
            ch_keys = r.keys(f"ack_{topic}_*")
            for key in ch_keys:
                data = r.get(key)
                if data:
                    q = RedisQueue(r, topic)
                    t, _data = json.loads(data)
                    if time.time() - t > 10 * 60:
                        r.delete(key)
                        q.put(_data)
                        del q
            if len(ch_keys) == 0:
                time.sleep(10)
            time.sleep(1)

    def stop(self):
        self.switch = 0


class RedisCh:
    def __init__(self, r, topic, data):
        self.r = r
        self.topic = topic
        self.id = uuid.uuid4()
        r.set(f"ack_{topic}_{self.id}", json.dumps([time.time(), data.decode()]))

    def basic_ack(self):
        self.r.delete(f"ack_{self.topic}_{self.id}")


if __name__ == '__main__':
    r1 = redis.Redis()
    r1.delete('test')
    # a = RedisQueue("test")
    # a.put(1)
    # print(type(a.get()))
    # r.set('test', 1)
    # print(r.get('test'), type(r.get('test').decode('utf8')))
    # RedisQueue.put(value=1, topic="test")
